/*********************************************************************
 * Webserver Hardware Profile Header File
 *
 * Author: Craig A. Lindley
 * Last Update: 11/05/2007
 ********************************************************************/

#ifndef __HARDWARE_PROFILE_H
#define __HARDWARE_PROFILE_H

#include "p18f2620.h"

// Define the clock frequency value.
// This value is used to calculate Tick Counter value
#define CLOCK_FREQ		(20000000ul)      // Hz

// Hardware mappings for custom web server hardware

// Onboard UART
#define UARTTX_TRIS			(TRISCbits.TRISC6)
#define UARTTX_IO			(PORTCbits.RC6)
#define UARTRX_TRIS			(TRISCbits.TRISC7)
#define UARTRX_IO			(PORTCbits.RC7)

// Onboard LEDs
#define LED0_TRIS			(TRISCbits.TRISC0)
#define LED0_IO				(PORTCbits.RC0)
#define LED1_TRIS			(TRISCbits.TRISC1)
#define LED1_IO				(PORTCbits.RC1)
#define LED2_TRIS			(TRISCbits.TRISC2)
#define LED2_IO				(PORTCbits.RC2)

// ENC28J60 I/O pins
#define ENC_RST_TRIS		(TRISBbits.TRISB4)
#define ENC_RST_IO			(LATBbits.LATB4)
#define ENC_CS_TRIS			(TRISBbits.TRISB3)
#define ENC_CS_IO			(LATBbits.LATB3)
#define ENC_SCK_TRIS		(TRISCbits.TRISC3)
#define ENC_SDI_TRIS		(TRISCbits.TRISC4)
#define ENC_SDO_TRIS		(TRISCbits.TRISC5)

#define ENC_SPI_IF			(PIR1bits.SSPIF)
#define ENC_SSPBUF			(SSPBUF)
#define ENC_SPISTAT			(SSP1STAT)
#define ENC_SPISTATbits		(SSPSTATbits)
#define ENC_SPICON1			(SSPCON1)
#define ENC_SPICON1bits		(SSPCON1bits)

// USBwiz I/O pins
#define USBW_CS_TRIS		(TRISAbits.TRISA0)
#define USBW_CS_IO			(LATAbits.LATA0)
#define USBW_DATARDY_TRIS	(TRISAbits.TRISA1)
#define USBW_DATARDY_IO		(PORTAbits.RA1)
#define USBW_BUSY_TRIS		(TRISAbits.TRISA2)
#define USBW_BUSY_IO		(PORTAbits.RA2)
#define USBW_RST_TRIS		(TRISAbits.TRISA3)
#define USBW_RST_IO			(LATAbits.LATA3)

// Function prototypes
#define BusyUART()				BusyUSART()
#define CloseUART()				CloseUSART()
#define ConfigIntUART(a)		ConfigIntUSART(a)
#define DataRdyUART()			DataRdyUSART()
#define OpenUART(a,b,c)			OpenUSART(a,b,c)
#define ReadUART()				ReadUSART()
#define WriteUART(a)			WriteUSART(a)
#define getsUART(a,b,c)			getsUSART(b,a)
#define putsUART(a)				putsUSART(a)
#define getcUART()				ReadUSART()
#define putcUART(a)				WriteUSART(a)
#define putrsUART(a)			putrsUSART((const rom char*)a)

#endif
