 /*********************************************************************
 *  Microchip TCP/IP Stack Configuration Header File
 *
 * Author: Craig A. Lindley
 * Last Update: 11/11/2007
 ********************************************************************/

#ifndef __TCPIPCONFIG_H
#define __TCPIPCONFIG_H

#include "TCPIP Stack/TCPIP.h"

// Modules to include in this project
#define STACK_USE_HTTP2_SERVER
#define STACK_USE_DHCP_CLIENT
#define STACK_USE_UDP
#define STACK_USE_TCP
#define STACK_CLIENT_MODE
#define STACK_USE_DNS

//
// TCP and UDP protocol options
//

#if defined(STACK_USE_TCP)
	// Allocate how much total RAM (in bytes) you want to allocate 
	// for use by your TCP TCBs, RX FIFOs, and TX FIFOs.  
	// Sockets can be scattered across several different storage 
	// mediums if you are out of space on one medium.
	#define TCP_ETH_RAM	0
	#define TCP_ETH_RAM_BASE_ADDRESS			(BASE_TCB_ADDR)
	#define TCP_ETH_RAM_SIZE					3264
	#define TCP_PIC_RAM	1
	#define TCP_PIC_RAM_BASE_ADDRESS			((WORD)&TCPBufferInPIC[0])
	#define TCP_PIC_RAM_SIZE					0
	#define TCP_SPI_RAM	2
	#define TCP_SPI_RAM_BASE_ADDRESS			(0x0000u)
	#define TCP_SPI_RAM_SIZE					0

	// These are the different types of TCP sockets 
	// that you want to use.  Each different type can have a 
	// different RX FIFO size, TX FIFO size, and even be stored in 
	// a different physical memory medium, optimizing storage
	#define TCP_PURPOSE_DEFAULT					0
	#define TCP_PURPOSE_GENERIC_TCP_CLIENT		1
	#define TCP_PURPOSE_GENERIC_TCP_SERVER		2
	#define TCP_PURPOSE_TELNET					3
	#define TCP_PURPOSE_FTP_COMMAND				4
	#define TCP_PURPOSE_FTP_DATA				5
	#define TCP_PURPOSE_TCP_PERFORMANCE_TEST	6
	#define TCP_PURPOSE_UART_2_TCP_BRIDGE		7
	#define TCP_PURPOSE_MP3_CLIENT				8
	
	#if defined(__TCP_C)
		// Define how many sockets are needed, what type they are,
		// where their TCB, TX FIFO, and RX FIFO should be stored, 
		// and how big the RX and TX FIFOs should be.  Making this 
		// initializer bigger or smaller defines how many total TCP
		// sockets are available.
		// Each socket requires 28 bytes of PIC RAM and 
		// 39+(TX FIFO size)+(RX FIFO size) bytes bytes of 
		// TCP_*_RAM each.
		// Note: The RX FIFO must be at least 1 byte in order to 
		// receive SYN and FIN messages required by TCP.  The TX 
		// FIFO can be zero if desired.
		ROM struct
		{
			BYTE vSocketPurpose;
			BYTE vMemoryMedium;
			WORD wTXBufferSize;
			WORD wRXBufferSize;
		} TCPSocketInitializer[] = 
		{
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
			{TCP_PURPOSE_DEFAULT,				TCP_ETH_RAM, 200, 80},
		};

		// If PIC RAM is used to store TCP socket FIFOs and TCBs, 
		// let's allocate it so the linker dynamically chooses 
		// where to locate it and prevents other variables from 
		// overlapping with it
		#if TCP_PIC_RAM_SIZE > 0
			static BYTE TCPBufferInPIC[TCP_PIC_RAM_SIZE] __attribute__((far));
		#endif
	#endif
#else
	// Don't allocate any RAM for TCP if TCP module isn't enabled
	#define TCP_ETH_RAM_SIZE 0
	#define TCP_PIC_RAM_SIZE 0
	#define TCP_SPI_RAM_SIZE 0
#endif


// Maximum avaialble UDP Sockets
#define MAX_UDP_SOCKETS     (9ul)
#define UDP_USE_TX_CHECKSUM		// This slows UDP TX performance by nearly 50%

// HTTP2 Server options
//
// Maximum numbers of simultaneous HTTP connections allowed.
// Each connection consumes 2 bytes of RAM and a TCP socket
#define MAX_HTTP_CONNECTIONS	(10ul)

// Indicate what file to serve when no specific one is requested
#define HTTP_DEFAULT_FILE		"index.htm"
#define HTTP_DEFAULT_LEN		9			//length of above

#endif
