// Modified version of USBwiz library
// Modified by Craig Lindley
// Last Update: 11/20/2007

#include <string.h>
#include "TCPIP Stack/USBwiz_lib.h"

//---------------------------------------------
int8 ASCIItoNibble(int8 digit)
{
	if ( digit<='9' )
    {
        return(digit-'0');
    }else if (digit>='A'&& digit<='F')
    {
        return((digit-'A')+10);
    }else
    {
        if(digit>='a' && digit<='f')
        {
            return((digit-'a')+10);
        }
    }
    return 0;
}

//---------------------------------------------
int32 HEXstrToDouble(int8 str[])
{
    int32 result;
    int8 i;

    result=0;
    i=0;
    while(str[i])
    {
        result=result<<4;
        if( ((str[i]>='0')&&(str[i]<='9')) )
          result=result | (str[i]-'0');
        if( ((str[i]>='A')&&(str[i]<='F'))  )
          result=result | ((str[i]-'A')+10);
        i++;
    }
    return result;
}

//---------------------------------------------
void GHI_PutNibble(int8 n)
{
	n&=0x0f;
	if(n<10)
		GHI_PutC((n+'0'));
	else
		GHI_PutC((n-10)+'A');
}

//---------------------------------------------
void GHI_PutDWORD(int32 dw)
{
	// send the high nibble first
	GHI_PutNibble(dw>>28);
	GHI_PutNibble(dw>>24);
	GHI_PutNibble(dw>>20);
	GHI_PutNibble(dw>>16);
	GHI_PutNibble(dw>>12);
	GHI_PutNibble(dw>>8);
	GHI_PutNibble(dw>>4);
	GHI_PutNibble(dw);
}

//---------------------------------------------
// the return data is always in this format
// !zz<CR>
int8 GHI_GetResult(void)
{
	int8 result;
	int8 c;

	c=GHI_GetC();
	if(c=='!')// the command has returned the results
	{
		result=ASCIItoNibble(GHI_GetC())<<4;
		result|=ASCIItoNibble(GHI_GetC());
		if(GHI_GetC()!='\r') // data MUST end with <CR>
			return ERROR_UNEXPECTED_VALUE;
		// everything went well
		return result;
	}
	return ERROR_UNEXPECTED_VALUE;	
}
//----------------------------------------------
int16 GHI_GetLibraryVersion()
{
	return GHI_USBWIZ_LIBRARY_VERSION;
}

//--------------------------------------------------------
int8 GHI_MountFATFileSystem(int8 device)
{
	int8 cmd[] = "FM ", c;
	
	GHI_PutS(cmd);
	GHI_PutC(device);
	GHI_PutC('\r');
	
	c = GHI_GetResult();
	if(c)		// if error
	{
		return c;
	}
	
	// ignore LUNs
	if(device == USB_DEVICE_PORT_0 || device == USB_DEVICE_PORT_1)
	{
		if(GHI_GetC() != '$')	// read data
			return ERROR_UNEXPECTED_VALUE;
			
		GHI_GetC();	// ignore
		GHI_GetC();	// ignore
		
		if(GHI_GetC() != '\r')
			return ERROR_UNEXPECTED_VALUE;

		c = GHI_GetResult();
	}
	return c;
}

//----------------------------------------------
int8 GHI_ChangeDirectory(int8 * filename)
{
	int8 cmd[]="CD ";
	
	GHI_PutS(cmd);
	GHI_PutS(filename);
	GHI_PutC('\r');
	return GHI_GetResult();
}

//-----------------------------------------------------------------------------------------
int8 GHI_OpenFile(int8 filehandle, int8 *filename, int8 openmode)
{
	int8 cmd[]="OF ";
	
	GHI_PutS(cmd);
	GHI_PutNibble(filehandle);
	GHI_PutC(openmode);
	GHI_PutC('>');
	GHI_PutS(filename);
	GHI_PutC('\r');
	return GHI_GetResult();
}
//-----------------------------------------------------------------------------------------
int8 GHI_SendReadCommand(int8 filehandle, int32 desireddatasize, int8 filler)
{
	int8 cmd[]="RF ";
	
	GHI_PutS(cmd);
	GHI_PutNibble(filehandle);
	GHI_PutC(filler);
	GHI_PutC('>');
	GHI_PutDWORD(desireddatasize);
	GHI_PutC('\r');

	return GHI_GetResult();
}
//-----------------------------------------------------------------------------------------
int8 GHI_GetReadAndWriteResult(int32 * actualdatasize)
{
	int8 datastring[9];
	int8 c;

	if(GHI_GetC()!='$')
		return ERROR_UNEXPECTED_VALUE;

	for(c=0;c<8;c++)
		datastring[c]=GHI_GetC();
	if(GHI_GetC()!='\r')
		return ERROR_UNEXPECTED_VALUE;
	datastring[8]=0;//terminate with null

	*actualdatasize=HEXstrToDouble(datastring);

	return GHI_GetResult();
}

//-----------------------------------------------------------------------------------------
int8 GHI_CloseFile(int8 filehandle)
{
	int8 cmd[]="CF ";
	
	GHI_PutS(cmd);
	GHI_PutNibble(filehandle);
	GHI_PutC('\r');

	return GHI_GetResult();
}


//-----------------------------------------------------------------------------------------
int8 GHI_SendReadWriteFileCommand(int8 readhandle, int8 writehandle,int32 size)
{
	int8 c;
	int8 temp[9];

	int8 cmd[]="RW ";
	
	GHI_PutS(cmd);
	GHI_PutNibble(readhandle);
	GHI_PutC(' ');
	GHI_PutNibble(writehandle);
	GHI_PutC('>');
	GHI_PutDWORD(size);
	GHI_PutC('\r');

	return GHI_GetResult();
}

//-----------------------------------------------------------------------------------------
int8 GHI_SeekFile(int8 filehandle, int32 newposition)
{
	int8 cmd[]="PF ";
	
	GHI_PutS(cmd);
	GHI_PutNibble(filehandle);
	GHI_PutC('>');
	GHI_PutDWORD(newposition);
	GHI_PutC('\r');

	return GHI_GetResult();
}

//-----------------------------------------------------------------------------------------
int8 GHI_GetFileInfo(int8 * filename, int32 * size, int8 * attributes, int32 * DateTime)
{
	int8 temp[9];
	int8 c;
	
	int8 cmd[]="IF ";
	
	GHI_PutS(cmd);
	GHI_PutS(filename);
	GHI_PutC('\r');
	c=GHI_GetResult();
	if(c)
	{
		// error code
		return c;
	}
	// the command passed
	if(GHI_GetC()!='$') return ERROR_UNEXPECTED_VALUE;
	for(c=0;c<8;c++)    
		temp[c]=GHI_GetC();
	temp[8]=0;//terminate
	*size=HEXstrToDouble(temp);
	
	if(GHI_GetC()!=' ')	return ERROR_UNEXPECTED_VALUE;
	if(GHI_GetC()!='$')	return ERROR_UNEXPECTED_VALUE;
	
	for(c=0;c<2;c++)// we got 2 bytes only
		temp[c]=GHI_GetC();
	temp[2]=0;//terminate
	
	*attributes=HEXstrToDouble(temp);// our function is smart enough to get just a byte back
	
	if(GHI_GetC()!=' ')	return ERROR_UNEXPECTED_VALUE;
	if(GHI_GetC()!='$')	return ERROR_UNEXPECTED_VALUE;
	
	for(c=0;c<8;c++)// we got 8 bytes only
		temp[c]=GHI_GetC();
	temp[8]=0;//terminate
	
	*DateTime=HEXstrToDouble(temp);// our function is smart enough to get just a byte back
	
	if(GHI_GetC()!='\r') return ERROR_UNEXPECTED_VALUE;
	
	return GHI_GetResult();
}

//---------------------------------------------
int8 GHI_ReadFile(int8 handle, int8 *buffer, int32 size, int8 filler, int32 *actualdatasize)
{
	int32 var32;
	if(var32 = GHI_SendReadCommand(handle, size, filler))
		return var32;
	
	for(var32 = 0; var32 < size; var32++)
	{
		*buffer++ = GHI_GetC();
	}
	return GHI_GetReadAndWriteResult(actualdatasize);
}
